%
% This script contains the solution to exercise @, studying the optimal
% parameter matching the material characteristic of a portal. 
%

load portal

% Try to enlarge these intervals, so you will see that the accuracy will
% decrease in the solution. 
dom1 = [ 4e9, 6e10 ]; % Young's modulus, admissible interval
dom2 = [ 2e3  3e3 ];  % mass density, admissible interval

% You might set this to false to accelerate the creation of the matrix
% function, clearly at the cost of the accuracy. A more refined model
% reduction approach would obviously work much better!
use_model_reduction = false;

% Model reduction strategy, for speeding up computations
if use_model_reduction
	xx = linspace(dom1(1), dom1(2), 3);
	yy = linspace(dom2(1), dom2(2), 3);

	B = zeros(size(K{1}, 1), length(xx) * length(yy) * 20);
	c = 0;
	for i = 1 : length(xx)
		for j = 1 : length(yy)
			x = xx(i); y = yy(j);
			[V, D] = eigs(K{1} + x * K{2} + y * K{3}, ...
						  M{1} + x * M{2} + y * M{3}, 20, 'SM');
			B(:, c+1:c+20) = V;
			c = c + 20;
		end
	end
	[U, S, V] = svd(B, 'econ');
	rk = sum( diag(S) > S(1,1) * 1e-3 );
	U = U(:, 1:rk);

	% Problem is symmetric -- project down
	for j = 1 : length(K)
		K{j} = U' * (K{j} * U);
		M{j} = U' * (M{j} * U);
	end
end

f2 = chebfun2(@(x,y) obj_fun(x,y, K, M, frequencies), ..., 
	[dom1, dom2], 'vectorize', 'off');

% Find the minimum point: 
[fopt, xopt] = min2(f2);

fprintf('The optimal Young''s modulus is: %e Pa\n', xopt(1));
fprintf('The optimal mass density is: %e kg/m³\n', xopt(2));

% f2 = sqrt(f2);

function r = obj_fun(x, y, K, M, frequencies)

	ff = eigs(K{1} + x * K{2} + y * K{3}, ...
			  M{1} + x * M{2} + y * M{3}, 5, 'SM');
	ff = sqrt(ff) / 2 / pi;
	r = norm(ff - frequencies).^2;

end