%
% This script contains the solution to exercise 3, studying the transient
% behavior of a solution of an ODE. 
%

load transient;

% Check that the matrix is stable 
if max(real(eig(A))) < 0
	fprintf('The matrix is stable\n');
else
	error('The matrix is unstable');
end

% All the starting configuration are given by a combination of x0 and x1
% which is still of norm 1, and therefore can be given as: 
%
% x = cos(s) * x0 + sin(s) * x1, 
%
% for some t in [0, 2 * pi].
x0 = [ zeros(5,1) ; 1 ; 0 ];
x1 = [ zeros(6,1) ; 1 ];

f2 = chebfun2(@(t, s) norm(expm(t*A) * (cos(s) * x0 + sin(s) * x1)).^2, ...
			  [0 10 0 2 * pi], 'vectorize', false);
		  
% Compute the maximum norm reached, and the starting configuration and time
% at which is reached. 
[f2max, f2x] = max2(f2);

fprintf('The maximum norm is %e, reached at time %e\n', sqrt(f2max), f2x(1));
fprintf('The starting configuration is x0 = \n\n');
disp(cos(f2x(2)) * x0 + sin(f2x(2)) * x1)

% Find the configuration that gives the minimum transient growth
f3 = max(f2, [], 2); % Maximum over the time interval
[f3min, f3x] = min(f3);

fprintf('The maximum norm for the optimal configuration is %e\n', sqrt(f3min));
fprintf('The starting configuration is x0 = \n\n');
disp(cos(f3x) * x0 + sin(f3x) * x1)
