%
% This script contains the solution to exercise 4, trying to approximate
% the action of an integral transform
%

f = chebfun(@(y) 1 + y); % Function to transform
k = @(x,y) log(1 + x.^2 + y.^2);
f2 = chebfun2(@(x,y) k(x,y) .* f(y));
fh = sum(f2, 1);

% Second test: we consider the non-separable kernel, and perform the
% recursive expansion; we note that this is not necessarily the best way to
% handle this problem, but shows how to handle chebfuns on subdomains. 

% This is the new kernel -- not needed because it is already built in the
% in the function integral_transform(). 
k = @(x,y) log(1 + abs(x - y));
ft = integral_transform(f, [-1 1 -1 1]);

subplot(1, 2, 1); plot(f); hold on; plot(fh);
subplot(1, 2, 2); plot(f); hold on; plot(ft);

function g = integral_transform(f, dom)
	% Try to see if the interval is small enough that we can approximate
	% the logarithm with a first-order expansion
	tol = 1/16;
	if max(dom([2 4])) - min(dom([1 3])) <= tol
		middle = mean(dom(3:4));
		gval = (f(middle) - f(dom(1))) * (middle - dom(1)).^2 / 2 + ...
			   (f(dom(2)) - f(middle)) * (dom(2) - middle).^2 / 2;
		g = chebfun(@(x) gval, dom(1:2)).';
	else
		xm = mean(dom(1:2));
		ym = mean(dom(3:4));
		
		g11 = integral_transform(f, [dom(1) xm dom(3) ym]);
		g22 = integral_transform(f, [xm dom(2) ym dom(4)]);
		
		I21 = chebfun2(@(x,y) log(1 + abs(x-y)) .* f(y), [dom(1) xm ym dom(4)]);
		I12 = chebfun2(@(x,y) log(1 + abs(x-y)) .* f(y), [xm dom(2) dom(3) ym]);
		
		g1 = g11 + sum(I21, 1);
		g2 = g22 + sum(I12, 1);
		
		g = chebfun(@(x) g1(x) .* (x <= xm) + g2(x) .* (x > xm), dom(1:2), 'splitting', 'on');
		g = g';
	end
end